'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

exports.default = function (_ref) {
	var t = _ref.types;

	return {
		pre: function pre() {
			// Used to track renaming imports in local file
			// eg. import { state as s } from 'cerebral/proxies';
			this.importedTagSet = new Set();
		},

		visitor: {
			ImportDeclaration: function ImportDeclaration(path) {
				var _path$node = path.node,
				    value = _path$node.source.value,
				    source = _path$node.source;

				if (t.isStringLiteral(source) && isValidImportLocation(value)) {
					// Verify that all imports are allowed and track the localName
					var _iteratorNormalCompletion = true;
					var _didIteratorError = false;
					var _iteratorError = undefined;

					try {
						for (var _iterator = path.node.specifiers[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
							var _step$value = _step.value,
							    _step$value$imported = _step$value.imported;
							_step$value$imported = _step$value$imported === undefined ? {} : _step$value$imported;
							var importName = _step$value$imported.name,
							    localName = _step$value.local.name;

							if (importName === undefined) {
								continue;
							}
							if (isAllowedImport(importName)) {
								this.importedTagSet.add(localName);
							} else {
								throw path.buildCodeFrameError('The Tag "' + importName + '" can\'t be imported');
							}
						}
						// Change import to the real 'cerebral/tags';
					} catch (err) {
						_didIteratorError = true;
						_iteratorError = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion && _iterator.return) {
								_iterator.return();
							}
						} finally {
							if (_didIteratorError) {
								throw _iteratorError;
							}
						}
					}

					path.node.source.value = 'cerebral/tags';
				}
			},
			MemberExpression: function MemberExpression(path) {
				// Always use the innermost MemberExpression
				if (!t.isIdentifier(path.node.object)) {
					return;
				}

				var tagName = path.node.object.name;

				if (!this.importedTagSet.has(tagName) || !isTagValidInThisScope(path.scope, tagName)) {
					return;
				}

				var quasi = [];
				var quasis = [quasi];
				var expressions = [];
				var prevWasExpression = false;

				var rootMemberExpression = void 0;
				var currentMember = path;

				// Iterate trough all parents
				while (t.isMemberExpression(currentMember)) {
					var _currentMember = currentMember,
					    _currentMember$node = _currentMember.node,
					    property = _currentMember$node.property,
					    computed = _currentMember$node.computed;

					// Plain id like state.a[1].b['test']

					if (isPlainPropertyAccess(t, property, computed)) {
						var value = t.isLiteral(property) ? property.value : property.name;
						quasi.push((quasi.length !== 0 || prevWasExpression ? '.' : '') + value);
						prevWasExpression = false;
						// Nested expressions like state.a[state.b]
					} else {
						quasi.push('.');
						expressions.push(property);
						quasi = [];
						quasis.push(quasi);
						prevWasExpression = true;
					}

					// Save the rootMember
					rootMemberExpression = currentMember;

					// Advance to next parent
					currentMember = currentMember.parentPath;
				}

				// Replace the rootMemberExpression
				// with the TaggedTemplate
				rootMemberExpression.replaceWith(t.taggedTemplateExpression(t.identifier(tagName), t.templateLiteral(quasis.map(function (v) {
					var str = v.join('');
					return t.templateElement({ raw: str, cooked: str });
				}), expressions)));
			}
		}
	};
};

var importNames = ['cerebral/proxies', 'cerebral-proxy-tags'];
var allowedImports = ['state', 'props', 'input', 'signal'];

function isValidImportLocation(location) {
	return importNames.indexOf(location.toLowerCase()) >= 0;
}

function isAllowedImport(importName) {
	return allowedImports.indexOf(importName.toLowerCase()) >= 0;
}

function isTagValidInThisScope(scope, name) {
	var binding = scope.getBinding(name);
	return binding && binding.kind === 'module';
}

function isPlainPropertyAccess(t, property, computed) {
	return t.isIdentifier(property) && computed === false || t.isLiteral(property) && property.value;
}